@@ -28,20 +28,6 @@
 /*
   2D Graphics library
 */
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <memory.h>
-#include <unistd.h>
-#include <ctype.h>
-
-#include <SDL/SDL.h>
-
-#ifdef HILDON        
-#include <SDL/SDL_syswm.h>
-#include <X11/Xutil.h>
-#endif
-
 #include "2d_graph.h"
 
 // -------------------------------------------------------
@@ -344,7 +330,7 @@ void sprite_store::sprite_flag_clear(tflag flag, spr_handle first, spr_handle nu
     p_sprites[first].flag_clear(flag);
 }
 
-spr_handle sprite_store::sprite_insert(char *p_file, spr_handle first, spr_handle num, spr_handle *p_last)
+spr_handle sprite_store::sprite_insert(const char *p_file, spr_handle first, spr_handle num, spr_handle *p_last)
 {  
   spr_handle i, j, max;
   int x, y, w, h;
@@ -479,21 +465,7 @@ SDL_Surface * graph_2d::create_screen(int flag, int width, int height, int bpp,
     fprintf(stderr, "unable to init SDL: %s", SDL_GetError());
     exit(0);
   }      
-
-#ifdef HILDON        
-	// to set the title in the task bar
-  SDL_SysWMinfo info;
-  SDL_VERSION(&info.version);
-  if(SDL_GetWMInfo(&info)) {
-    Display *dpy = info.info.x11.display;
-    Window win;
-    win = info.info.x11.fswindow;
-    XStoreName(dpy, win, "Berusky");
-    win = info.info.x11.wmwindow;
-    XStoreName(dpy, win, "Berusky");
-  }
-#endif
-	
+        
   flag |= SDL_HWSURFACE;
   
   if(fullscreen)
@@ -691,7 +663,7 @@ void font::print(char *p_string, RECT *p_res, int lines)
   ::free(p_start);
 }
 
-void font::print(RECT *p_res, tpos x, tpos y, char *p_text,...)
+void font::print(RECT *p_res, tpos x, tpos y, const char *p_text,...)
 {
   char      text[2000];
   va_list   arguments;  
@@ -707,7 +679,7 @@ void font::print(RECT *p_res, tpos x, tpos y, char *p_text,...)
   print(text,p_res);
 }
 
-void font::print(RECT *p_res, tpos x, tpos y, int lines, char *p_text,...)
+void font::print(RECT *p_res, tpos x, tpos y, int lines, const char *p_text,...)
 {
   char      text[2000];
   va_list   arguments;  
