@@ -42,6 +42,20 @@
 
 #include <assert.h>
 #include <limits.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <memory.h>
+#include <ctype.h>
+
+#include "portability.h"
+
+#ifdef LINUX
+#include <SDL/SDL.h>
+#endif
+
+#ifdef WINDOWS
+#include "SDL.h"
+#endif
 
 #include "ini.h"
 #include "utils.h"
@@ -116,7 +130,7 @@ public:
     strncpy(graphics_dir,p_dir,MAX_FILENAME);
   }
 
-  static char * graphics_dir_get(void)
+  static const char * graphics_dir_get(void)
   {
     return(graphics_dir);
   }
@@ -369,7 +383,7 @@ public:
     *******************************************************
   */
 
-  spr_handle sprite_insert(char *p_file, spr_handle first, spr_handle num = 1, spr_handle *p_last = NULL);
+  spr_handle sprite_insert(const char *p_file, spr_handle first, spr_handle num = 1, spr_handle *p_last = NULL);
   spr_handle sprite_insert(surf_handle sf_handle, tflag flag_ = 0, RECT *p_rect = NULL, spr_handle first = INSERT_APPEND);
   spr_handle sprite_insert(SURFACE *p_surf, tflag flag_ = 0, RECT *p_rect = NULL, spr_handle first = INSERT_APPEND);
   spr_handle sprite_insert(SPRITE *p_spr, int num = 1, spr_handle first = INSERT_APPEND);
@@ -447,7 +461,7 @@ public:
     surface::graphics_dir_set(p_dir);
   }
 
-  char * graphics_dir_get(void)
+  const char * graphics_dir_get(void)
   {
     return(surface::graphics_dir_get());
   }
@@ -486,7 +500,7 @@ public:
     return((p_screen->get_height() - store.sprite_get_height(spr)) / 2);
   }
 
-  spr_handle sprite_insert(char *p_file, spr_handle first, spr_handle num = 1, spr_handle *p_last = NULL)
+  spr_handle sprite_insert(const char *p_file, spr_handle first, spr_handle num = 1, spr_handle *p_last = NULL)
   {
     return(store.sprite_insert(p_file,first,num,p_last));
   }
@@ -640,6 +654,10 @@ public:
 
   graph_2d(tpos dx, tpos dy, int depth, bool fullscreen) : store(SURFACES, SPRITES), rect_last(0)
   {
+    /* sdl init */
+    bprintf("SDL Init...");
+    SDL_Init(SDL_INIT_VIDEO);
+
      /* Create new screen */
     bprintf("Seting up screen %dx%d, color depth %d bits, fullscreen = %d...",dx, dy, depth, fullscreen);
     create_screen(0, dx, dy, depth, fullscreen);
@@ -883,8 +901,8 @@ public:
   }
 
   void print(char *p_string, RECT *p_res = NULL, int lines = 0);
-  void print(RECT *p_res, tpos x, tpos y, char *p_text,...);
-  void print(RECT *p_res, tpos x, tpos y, int lines, char *p_text,...);  
+  void print(RECT *p_res, tpos x, tpos y, const char *p_text,...);
+  void print(RECT *p_res, tpos x, tpos y, int lines, const char *p_text,...);  
 
 } FONT;
 
