@@ -30,7 +30,6 @@
 #include <stdlib.h>
 #include <time.h>
 #include <string.h>
-#include <curses.h>
 
 #include "berusky.h"
 #include "berusky_gui.h"
@@ -88,29 +87,6 @@ char * berusky::levelset_get_passwd(int level)
   return(store.levelset_get_passwd(level));
 }
 
-#ifdef HILDON
-char * berusky::levelset_get_name(int level)
-{
-  return(store.levelset_get_name(level));
-}
-
-char * berusky::levelset_get_difficulty(int level_set)
-{
-  static char *p_name_table[5] = {NULL,NULL,NULL,NULL,NULL};  
-
-  if(!p_name_table[0]) {
-    p_name_table[0] = _("training");
-    p_name_table[1] = _("easy");
-    p_name_table[2] = _("intermediate");
-    p_name_table[3] = _("advanced");
-    p_name_table[4] = _("impossible");
-  }
-
-  assert(level_set < (int)(sizeof(p_name_table)));
-  return(p_name_table[level_set]);
-}
-#endif
-
 char * berusky::levelset_get_difficulty(void)
 {
   static char *p_name_table[5] = {NULL,NULL,NULL,NULL,NULL};  
@@ -144,6 +120,11 @@ bool berusky::levelset_search(char *p_passwd, int *p_set, int *p_level)
   return(FALSE);
 }
 
+void berusky::levelset_set_level( int level )
+{
+  state.level_num = level;
+}
+
 int  berusky::levelset_get_level(void)
 {
   return(state.level_num);
