@@ -43,16 +43,26 @@ using namespace std;
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
-#include <syslog.h>
 #include <string.h>
-
-#include <libintl.h>
 #include <locale.h>
 
+#include "portability.h"
+
+#ifdef LINUX
+
+#include <syslog.h>
+#include <libintl.h>
 #include <SDL/SDL.h>
 
 #define _(string) gettext (string)
 
+#elif WINDOWS
+
+#include "SDL.h"
+#define _(string) (string)
+
+#endif
+
 /* Basic header files
 */
 #include "defines.h"
@@ -142,7 +152,8 @@ public:
   bool   level_load(char *p_level);
   bool   levelset_load(int set);
   bool   levelset_is_custom(void);
-  
+
+  void   levelset_set_level(int level);
   int    levelset_get_level(void);
   int    levelset_get_set(void);
   int    levelset_get_levelnum(void);
@@ -167,11 +178,6 @@ public:
 
   BERUSKY_STATE status_get(void);  
 
-#ifdef HILDON
-  char * berusky::levelset_get_name(int level);
-  char * berusky::levelset_get_difficulty(int leve_set);
-#endif
-
 } BERUSKY;
 
 #endif
