@@ -570,8 +570,8 @@ through."));
 
             p_font->start_set(20, 140);
             p_font->print(_("one-pass door - can be used only once,\n\
-then they close them off and there's no\n\
-way to open them\n"));
+then it is closed off and there's no\n\
+way to open it\n"));
 
             x = 40;
             y = 220;
@@ -664,6 +664,7 @@ which can be controlled by these keys:"));
         int start_y;
       
         p_font->print(NULL, 20,  start_y = Y_START, _("arrows"));
+        p_font->print(NULL, 20,  start_y += Y_DIFF, _("SHIFT+arrows"));
         
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("Tab"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("N"));
@@ -673,7 +674,8 @@ which can be controlled by these keys:"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("F2"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("F3"));
       
-        p_font->print(NULL, 69,  start_y = Y_START, _(" . . . . . . . move the bug"));
+        p_font->print(NULL, 135,  start_y = Y_START, _(". . . . . move the bug"));        
+        p_font->print(NULL, 205,  start_y += Y_DIFF, _(". . quick bug movement"));
         p_font->print(NULL, 86,  start_y += Y_DIFF, _(". . . . . . . switch among the bugs"));
         p_font->print(NULL, 60,  start_y += Y_DIFF, _(". . . . . . . . change the music"));
         p_font->print(NULL, 60,  start_y += Y_DIFF, _(". . . . . . . . demo"));
@@ -835,7 +837,7 @@ void game_gui::menu_level_hint(MENU_STATE state, int data, int data1)
         if(p_hint) {
           p_font->print(NULL, 20, 100, _("Level hint:"));
         } else {
-          p_hint = _("\nSory dude, no hint available for this\nlevel.");
+          p_hint = _("\nSorry dude, no hint available for this\nlevel.");
         }
         p_font->print(NULL, 20, 120, p_hint);
       
