@@ -29,68 +29,32 @@
 #include <stdlib.h>
 #include <time.h>
 #include <string.h>
-#include <curses.h>
-#include <sys/wait.h>
 #include <error.h>
 #include <errno.h>
+#include <stdio.h>
+
+#include "portability.h"
+
+#ifdef LINUX
+#include <sys/wait.h>
+#elif WINDOWS
+#include <stdio.h>
+#include <process.h>
+#endif
 
 #include "berusky.h"
 #include "berusky_gui.h"
 #include "main.h"
 #include "editor.h"
 
-ITEM_REPOSITORY *item_panel::p_repo = NULL;
+ITEM_REPOSITORY *editor_panel::p_repo = NULL;
 
-// draws item panel, writes records to item_array and 
-// returns number of written items
-
-/* Rect is
-  start_x;
-  start_y;
-
-  dx = dx*item_num;
-  dy = dy*item_num;
-*/
-
-bool item_panel::panel_scroll_check(int direction)
-{
-  int tmp_item    = item_first    + item_increment * direction;
-  int tmp_variant = variant_first + variant_increment * direction;
-
-  if(tmp_item < 0 || tmp_variant < 0)
-    return(FALSE);
-
-  int i;
-  for(i = 0; i < item_num; i++) {
-    int item =    tmp_item + item_increment * i;
-    int variant = tmp_variant + variant_increment * i;
-  
-    if(!p_repo->item_valid(item) || !p_repo->variant_valid(item,variant))
-      break;
-  }
-  
-  return(i == item_num);
-}
-
-void item_panel::panel_scroll(int direction, EDITOR_SELECTION *p_sel, bool redraw)
-{
-  if(panel_scroll_check(direction)) {
-    item_first    += item_increment * direction;
-    variant_first += variant_increment * direction;
-    if(item_selected != NO_SELECTION) {
-      item_return(item_selected,p_sel,direction);
-    }
-    if(redraw)
-      publish();
-  }
-}
-
-int  item_panel::item_return(tpos x, tpos y, EDITOR_SELECTION *p_sel)
+bool editor_panel::slot_return(tpos x, tpos y, int &slot)
 {
   tpos sx = start_x;
   tpos sy = start_y;
 
-  /* Panel is horizontal/vertical? */
+  // Panel is horizontal/vertical?
   if(dx) { 
     sx += EDIT_ARROW_DX;
   } else {
@@ -98,60 +62,59 @@ int  item_panel::item_return(tpos x, tpos y, EDITOR_SELECTION *p_sel)
   }  
 
   int i;
-  for(i = 0; i < item_num; i++) {
-  
+  for(i = 0; i < panel_size_get(); i++) {
+    if(!slot_valid(i))
+      return(FALSE);
+    
     tpos tx = sx + dx*i;
     tpos ty = sy + dy*i;
-  
-    int item =    item_first + item_increment*i;
-    int variant = variant_first + variant_increment*i;
-  
-    if(!p_repo->item_valid(item) || !p_repo->variant_valid(item,variant))
-      return(NO_SELECTION);
     
     if(INSIDE(tx, x, ITEM_SIZE_X) && INSIDE(ty, y, ITEM_SIZE_Y)) {
-      if(p_sel) {
-        p_sel->item = item;
-        p_sel->variant = variant;      
-      }
-      return(i);
+      slot = i;
+      return(TRUE);
     }
   }
   
-  return(NO_SELECTION);
+  return(FALSE);
 }
 
-int  item_panel::item_return(int selected_item, EDITOR_SELECTION *p_sel, int correction)
-{
-  int i;
+void editor_panel::panel_scroll(int direction, EDITOR_SELECTION *p_sel, bool redraw)
+{  
+  int panel_item_first = item_firts_get();  
+  int panel_item_num = items_num_get();
+  int item_new;
 
-  selected_item += correction;
-  if(selected_item < 0)
-    selected_item = 0;
-  if(selected_item >= item_num)
-    selected_item = item_num-1;
-
-  for(i = 0; i < item_num; i++) {
+  if(direction == ITEMS_START) {
+    item_new = 0;
+  }
+  else if(direction == ITEMS_END) {
+    item_new = panel_item_num - panel_size_get();
+  }
+  else {  
+    item_new = panel_item_first + direction;
+  }
   
-    int item =    item_first + item_increment*i;
-    int variant = variant_first + variant_increment*i;
+  if(item_new < 0) {
+    item_new = 0;
+  }
   
-    if(!p_repo->item_valid(item) || !p_repo->variant_valid(item,variant)) {
-      return(i > 0 && i == selected_item ? i-1 : NO_SELECTION);
-    }
-    if(i == selected_item) {
-      if(p_sel) {
-        p_sel->item = item;
-        p_sel->variant = variant;      
-      }
-      return(i);
+  if(item_new == panel_item_first) {
+    // no scroll - don't change it
+    return;
+  }
+  
+  // Do we scroll the panel?
+  if(panel_item_num > panel_size_get()) {
+    if(item_new + panel_size_get() > panel_item_num) {
+      item_new = panel_item_num - panel_size_get();
+      assert(item_new >= panel_item_first);
     }
+    if(item_new != panel_item_first)
+      panel_set(item_new, p_sel, TRUE, redraw);
   }
-
-  return(NO_SELECTION);
 }
 
-void item_panel::publish(void)
+RECT editor_panel::boundary_get(void)
 {
   RECT r;
 
@@ -164,70 +127,7 @@ void item_panel::publish(void)
   
     r.x = sx;
     r.y = sy;
-    r.w = dx*item_num;
-    r.h = ITEM_SIZE_Y;  
-  } else {
-    // Panel is vertical
-    sy += EDIT_ARROW_DY;
-  
-    r.x = sx;
-    r.y = sy;
-    r.w = ITEM_SIZE_X;
-    r.h = dy*item_num;
-  }
-
-  p_grf->fill(&r,0);
-
-  int i;
-  for(i = 0; i < item_num; i++) {
-  
-    tpos x = sx + dx*i;
-    tpos y = sy + dy*i;
-  
-    int item =    item_first + item_increment*i;
-    int variant = variant_first + variant_increment*i;
-  
-    if(!p_repo->item_valid(item) || !p_repo->variant_valid(item,variant))
-      break;
-
-    #define COLOR_HIGHLIGHTED_R   0
-    #define COLOR_HIGHLIGHTED_G   0
-    #define COLOR_HIGHLIGHTED_B   200
-    
-    #define COLOR_SELECTED_R      175
-    #define COLOR_SELECTED_G      0
-    #define COLOR_SELECTED_B      0
-    
-    if(i == item_highlighted) {
-      p_grf->fill(x,y,ITEM_SIZE_X,ITEM_SIZE_Y,p_grf->color_map(COLOR_HIGHLIGHTED_R,
-                  COLOR_HIGHLIGHTED_G, COLOR_HIGHLIGHTED_B));
-    }
-    if(i == item_selected) {
-      p_grf->fill(x,y,ITEM_SIZE_X,ITEM_SIZE_Y, p_grf->color_map(COLOR_SELECTED_R,
-                  COLOR_SELECTED_G, COLOR_SELECTED_B));
-    }
-    
-    p_repo->draw(x+CELL_SIZE_X,y+CELL_SIZE_X,item,variant,0);
-  }
-
-  p_grf->redraw_add(&r);
-}
-
-// draws controls for a panel
-void item_panel::publish_controls(INPUT *p_input)
-{
-  RECT r;
-
-  tpos sx = start_x;
-  tpos sy = start_y;
-
-  if(dx) {
-    // Panel is horizontal
-    sx += EDIT_ARROW_DX;
-  
-    r.x = sx;
-    r.y = sy;
-    r.w = dx*item_num;
+    r.w = dx*panel_size_get();
     r.h = ITEM_SIZE_Y;
   } else {
     // Panel is vertical
@@ -236,9 +136,16 @@ void item_panel::publish_controls(INPUT *p_input)
     r.x = sx;
     r.y = sy;
     r.w = ITEM_SIZE_X;
-    r.h = dy*item_num;
+    r.h = dy*panel_size_get();
   }
 
+  return r;
+}
+
+void editor_panel::register_controls_events(INPUT *p_input)
+{
+  RECT r = boundary_get();
+
   /* Scrolling (by mouse wheel) */
   p_input->mevent_add(MOUSE_EVENT(MOUSE_STATE(r,MASK_WHEEL_UP),
                       MEVENT_ACTIVATE_ONCE|MEVENT_MOUSE_IN|MEVENT_MOUSE_BUTTONS,
@@ -246,7 +153,7 @@ void item_panel::publish_controls(INPUT *p_input)
   p_input->mevent_add(MOUSE_EVENT(MOUSE_STATE(r,MASK_WHEEL_DOWN),
                       MEVENT_ACTIVATE_ONCE|MEVENT_MOUSE_IN|MEVENT_MOUSE_BUTTONS,
                       LEVEL_EVENT(ED_LEVEL_IPANEL_SCROLL, panel_handle, 1)));
-
+                      
   /* Highlight of items in panel */
   p_input->mevent_add(MOUSE_EVENT(MOUSE_STATE(r),
                       MEVENT_MOUSE_EXTERNAL|MEVENT_MOUSE_IN, 
@@ -260,16 +167,12 @@ void item_panel::publish_controls(INPUT *p_input)
 
   if(dx) { // Panel is horizontal
     {
-      p_grf->draw(EDIT_ARROW_LEFT,start_x,start_y+CELL_SIZE_Y);
-      p_grf->redraw_add(EDIT_ARROW_LEFT,start_x,start_y+CELL_SIZE_Y);
       RECT t = {start_x, start_y+CELL_SIZE_Y, EDIT_ARROW_DX, EDIT_ARROW_DY};
       p_input->mevent_add(MOUSE_EVENT(MOUSE_STATE(t,MASK_BUTTON_LEFT),
                           MEVENT_MOUSE_IN|MEVENT_MOUSE_BUTTONS, LEVEL_EVENT(ED_LEVEL_IPANEL_SCROLL,
                           panel_handle,-1)));
     }
     {
-      p_grf->draw(EDIT_ARROW_RIGHT,r.x+r.w,r.y+CELL_SIZE_Y);
-      p_grf->redraw_add(EDIT_ARROW_RIGHT,r.x+r.w,r.y+CELL_SIZE_Y);
       RECT t = {r.x+r.w,r.y+CELL_SIZE_Y, EDIT_ARROW_DX, EDIT_ARROW_DY};
       p_input->mevent_add(MOUSE_EVENT(MOUSE_STATE(t,MASK_BUTTON_LEFT),
                           MEVENT_MOUSE_IN|MEVENT_MOUSE_BUTTONS, LEVEL_EVENT(ED_LEVEL_IPANEL_SCROLL,
@@ -277,16 +180,12 @@ void item_panel::publish_controls(INPUT *p_input)
     }
   } else { // Panel is vertical
     {
-      p_grf->draw(EDIT_ARROW_UP,start_x+CELL_SIZE_X,start_y);
-      p_grf->redraw_add(EDIT_ARROW_UP,start_x+CELL_SIZE_X,start_y);
       RECT t = {start_x+CELL_SIZE_X, start_y, EDIT_ARROW_DX, EDIT_ARROW_DY};
       p_input->mevent_add(MOUSE_EVENT(MOUSE_STATE(t,MASK_BUTTON_LEFT),
                           MEVENT_MOUSE_IN|MEVENT_MOUSE_BUTTONS, LEVEL_EVENT(ED_LEVEL_IPANEL_SCROLL,
                           panel_handle,-1)));
     }
     {
-      p_grf->draw(EDIT_ARROW_DOWN,r.x+CELL_SIZE_X,r.y+r.h);
-      p_grf->redraw_add(EDIT_ARROW_DOWN,r.x+CELL_SIZE_X,r.y+r.h);
       RECT t = {r.x+CELL_SIZE_X, r.y+r.h, EDIT_ARROW_DX, EDIT_ARROW_DY};
       p_input->mevent_add(MOUSE_EVENT(MOUSE_STATE(t,MASK_BUTTON_LEFT),
                           MEVENT_MOUSE_IN|MEVENT_MOUSE_BUTTONS, LEVEL_EVENT(ED_LEVEL_IPANEL_SCROLL,
@@ -295,9 +194,110 @@ void item_panel::publish_controls(INPUT *p_input)
   }  
 }
 
+void editor_panel::slot_draw(int slot, int item, int variant)
+{
+  RECT r = boundary_get();
+
+  tpos x = r.x + dx*slot;
+  tpos y = r.y + dy*slot;
+
+  p_grf->fill(x,y,ITEM_SIZE_X,ITEM_SIZE_Y,0);
+
+  if(!p_repo->item_valid(item) || !p_repo->variant_valid(item,variant))
+    return;
+  
+  #define COLOR_HIGHLIGHTED_R   0
+  #define COLOR_HIGHLIGHTED_G   0
+  #define COLOR_HIGHLIGHTED_B   200
+  
+  #define COLOR_SELECTED_R      175
+  #define COLOR_SELECTED_G      0
+  #define COLOR_SELECTED_B      0
+  
+  if(slot == panel_slot_highlighted) {
+    p_grf->fill(x,y,ITEM_SIZE_X,ITEM_SIZE_Y,p_grf->color_map(COLOR_HIGHLIGHTED_R,
+                COLOR_HIGHLIGHTED_G, COLOR_HIGHLIGHTED_B));
+  }
+  if(slot == panel_slot_selected) {
+    p_grf->fill(x,y,ITEM_SIZE_X,ITEM_SIZE_Y, p_grf->color_map(COLOR_SELECTED_R,
+                COLOR_SELECTED_G, COLOR_SELECTED_B));
+  }
+  
+  p_repo->draw(x+CELL_SIZE_X,y+CELL_SIZE_X,item,variant,0);
+}
+
+void editor_panel::controls_draw(void)
+{
+  RECT r = boundary_get();
+
+  if(dx) { // Panel is horizontal
+    {
+      p_grf->draw(EDIT_ARROW_LEFT,start_x,start_y+CELL_SIZE_Y);
+      p_grf->redraw_add(EDIT_ARROW_LEFT,start_x,start_y+CELL_SIZE_Y);
+    }
+    {
+      p_grf->draw(EDIT_ARROW_RIGHT,r.x+r.w,r.y+CELL_SIZE_Y);
+      p_grf->redraw_add(EDIT_ARROW_RIGHT,r.x+r.w,r.y+CELL_SIZE_Y);
+    }
+  } else { // Panel is vertical
+    {
+      p_grf->draw(EDIT_ARROW_UP,start_x+CELL_SIZE_X,start_y);
+      p_grf->redraw_add(EDIT_ARROW_UP,start_x+CELL_SIZE_X,start_y);
+    }
+    {
+      p_grf->draw(EDIT_ARROW_DOWN,r.x+CELL_SIZE_X,r.y+r.h);
+      p_grf->redraw_add(EDIT_ARROW_DOWN,r.x+CELL_SIZE_X,r.y+r.h);
+    }
+  }  
+}
+
+/* Select the select item by variant panel
+*/
+void item_panel::item_select(EDITOR_SELECTION *p_sel)
+{
+  int slot = selected_slot_get();
+  if(slot == NO_SELECTION)
+    return;
+  
+  int selected_item = panel_item_first+slot;
+  
+  // Propagate the selected item to variant panel
+  ((VARIANT_PANEL*)attached_panel_get())->panel_item_set(selected_item, FALSE);
+  ((VARIANT_PANEL*)attached_panel_get())->panel_set(0, p_sel, FALSE, TRUE);
+  ((VARIANT_PANEL*)attached_panel_get())->slot_select(0, p_sel, TRUE, TRUE);
+}
+
+void item_panel::panel_draw(void)
+{
+  RECT r = boundary_get();
+
+  p_grf->fill(&r,0);
+  int i;
+  for(i = 0; i < panel_size_get(); i++) {
+    slot_draw(i, panel_item_first + i, 0);
+  }
+  p_grf->redraw_add(&r);
+
+  controls_draw();
+}
+
+void variant_panel::panel_draw(void)
+{
+  RECT r = boundary_get();
+
+  p_grf->fill(&r,0);
+  int i;
+  for(i = 0; i < panel_size_get(); i++) {
+    slot_draw(i, panel_item, panel_variant_first + i);
+  }
+  p_grf->redraw_add(&r);
+
+  controls_draw();
+}
+
 editor_layer_config::editor_layer_config(void)
 {
-  
+
 }
 
 editor_gui::editor_gui(ITEM_REPOSITORY *p_repo_, DIR_LIST *p_dir_):
@@ -306,88 +306,97 @@ editor_gui::editor_gui(ITEM_REPOSITORY *p_repo_, DIR_LIST *p_dir_):
   p_repo(p_repo_),
   level(p_repo_), 
   console(&input,CONSOLE_X,CONSOLE_Y,CONSOLE_DX,CONSOLE_LINES),
-  sel(&level)
+  selected_editor_item(&level)
+{
+  editor_panel::set_up(p_repo_);
+
+  ipanel[0] = new ITEM_PANEL(ITEMS_IN_PANEL, VERTICAL, 0, 0, PANEL_HANDLE_1);
+  ipanel[1] = new VARIANT_PANEL(ITEMS_IN_PANEL, VERTICAL, ITEM_SIZE_X, 0, PANEL_HANDLE_2);
+  ipanel[2] = new ITEM_PANEL(ITEMS_IN_PANEL, HORIZONTAL, 2*ITEM_SIZE_X, 0, PANEL_HANDLE_3);
+  ipanel[3] = new VARIANT_PANEL(ITEMS_IN_PANEL, HORIZONTAL, 2*ITEM_SIZE_X, ITEM_SIZE_Y, PANEL_HANDLE_4);
+
+  ipanel[0]->attached_panel_set(ipanel[1]);
+  ipanel[2]->attached_panel_set(ipanel[3]);
+
+  ipanel[1]->attached_panel_set(ipanel[0]);
+  ipanel[3]->attached_panel_set(ipanel[2]);
+
+  editor_reset();
+}
+
+editor_gui::~editor_gui(void)
+{
+  int i;
+  for(i = 0; i < PANELS; i++) {    
+    delete ipanel[i];
+  }
+}
+
+void editor_gui::editor_reset(void)
 {
   input.mevent_clear();
   input.events_wait(TRUE);
 
+  draw_level = TRUE;
+
   // Clear whole screen
   p_grf->clear();
+
+  int i;
+  for(i = 0; i < PANELS; i++) {
+    ipanel[i]->register_controls_events(&input);
+  }
+  level_config();
   
-  // TODO -- predelat vsechno draw -> na konfigy (reset)
-  level_config();  
-  panel_reset();
+  panel_draw();
   selection_draw();
   layer_menu_draw();
   layer_status_draw();
   level_draw();
   layer_active_set(ALL_LEVEL_LAYERS);
   side_menu_create();
-  side_menu_update();
+  side_menu_draw();
 
   level.back_max_set(background_num(p_dir));
 }
 
-editor_gui::~editor_gui(void)
-{
-
-}
-
 // Panel interface
 void editor_gui::panel_item_select(int panel, tpos x, tpos y)
 {
   panel -= PANEL_HANDLE_1;
   assert(panel >= 0 && panel < PANELS);
 
+  EDITOR_PANEL *p_attached = ipanel[panel]->attached_panel_get();
+
   int i;
   for(i = 0; i < PANELS; i++) {
-    if(i == panel)
-      ipanel[i].item_select(x,y,&sel);
-    else
-      ipanel[i].item_unselect();    
+    if(p_attached != ipanel[i]) {
+      ipanel[i]->slot_unselect(TRUE,TRUE);
+    }
   }
+
+  // Select the selection
+  ipanel[panel]->slot_select(x,y,&selected_editor_item,TRUE,TRUE);
+  
   // Draw the current selection
   selection_draw();
 
   // Set active layer
-  layer_active_set(p_repo->item_get_layer(sel.item));
+  layer_active_set(p_repo->item_get_layer(selected_editor_item.item));
 }
 
 void editor_gui::panel_item_highlight(int panel, tpos x, tpos y)
 {
   panel -= PANEL_HANDLE_1;
   assert(panel >= 0 && panel < PANELS);
-  ipanel[panel].item_highlight(x,y);
+  ipanel[panel]->slot_highlight(x,y,TRUE);
 }
 
-void editor_gui::panel_reset(void)
+void editor_gui::panel_draw(void)
 {
-  #define ITEMS_X_START   0
-  #define ITEMS_Y_START   0
-  #define ITEMS_X_DIFF    0
-  #define ITEMS_Y_DIFF    35
-  #define ITEMS_DX        (EDITOR_RESOLUTION_X-ITEMS_X_START)
-  #define ITEMS_DY        (EDITOR_RESOLUTION_Y-ITEMS_Y_START)
-
-  //-- dodelat tu listu
-  #define ITEMS_IN_PANEL  12
-
-  item_panel::set_up(p_repo);
-
-  ipanel[0].set_up(ITEMS_IN_PANEL, VERTICAL, 0, 0, PANEL_HANDLE_1,ipanel+1);
-  ipanel[0].item_increment = 1;
-  ipanel[1].set_up(ITEMS_IN_PANEL, VERTICAL, ITEM_SIZE_X, 0, PANEL_HANDLE_2);
-  ipanel[1].variant_increment = 1;
-
-  ipanel[2].set_up(ITEMS_IN_PANEL, HORIZONTAL, 2*ITEM_SIZE_X, 0, PANEL_HANDLE_3, ipanel+3);
-  ipanel[2].item_increment = 1;
-  ipanel[3].set_up(ITEMS_IN_PANEL, HORIZONTAL, 2*ITEM_SIZE_X, ITEM_SIZE_Y, PANEL_HANDLE_4);
-  ipanel[3].variant_increment = 1;
-
   int i;
   for(i = 0; i < PANELS; i++) {
-    ipanel[i].publish();
-    ipanel[i].publish_controls(&input);
+    ipanel[i]->panel_draw();
   }
 }
 
@@ -395,10 +404,29 @@ void editor_gui::panel_scroll(int panel, int direction)
 {
   panel -= PANEL_HANDLE_1;
   assert(panel >= 0 && panel < PANELS);
-  ipanel[panel].panel_scroll(direction,&sel); 
+  ipanel[panel]->panel_scroll(direction,&selected_editor_item,TRUE);
   selection_draw();
 }
 
+void editor_gui::panel_scroll_mouse(int direction)
+{
+  MOUSE_STATE *p_state = input.mouse_state_get();
+  int panel = -1;
+
+  int i;
+  for(i = 0; i < PANELS; i++) {
+    RECT r = ipanel[i]->boundary_get();
+    if(p_state->in_rect(r)) {
+      panel = i;
+      break;
+    }
+  }
+
+  if(panel != -1) {  
+    panel_scroll(PANEL_HANDLE_1+panel, direction);
+  }
+}
+
 /*
 #define EDIT_ITEM_START_X        (EDIT_COORD_START_X)
 #define EDIT_ITEM_START_Y        (EDIT_COORD_START_Y+EDIT_COORD_DY)
@@ -415,8 +443,10 @@ void editor_gui::selection_draw(void)
   p_font->alignment_set(LEFT);
   p_grf->fill(EDIT_ITEM_START_X,EDIT_ITEM_START_Y,EDIT_ITEM_DX,EDIT_ITEM_DY,0);
   p_font->print(NULL,EDIT_ITEM_START_X,EDIT_ITEM_START_Y,_("I:%d V:%d R:%d L:%d"),
-                sel.item, sel.variant, sel.rotation, 
-                p_repo->item_get_layer(sel.item));
+                selected_editor_item.item, 
+                selected_editor_item.variant, 
+                selected_editor_item.rotation, 
+                p_repo->item_get_layer(selected_editor_item.item));
   p_grf->redraw_add(EDIT_ITEM_START_X,EDIT_ITEM_START_Y,EDIT_ITEM_DX,EDIT_ITEM_DY);
 
   {
@@ -431,7 +461,9 @@ void editor_gui::selection_draw(void)
   
     p_grf->fill(&r,p_grf->color_map(COLOR_BACK_R, COLOR_BACK_G, COLOR_BACK_B));
     p_repo->draw(EDIT_ITEM_PICT_START_X+CELL_SIZE_X,EDIT_ITEM_PICT_START_Y+CELL_SIZE_Y,
-                 sel.item, sel.variant, sel.rotation);
+                 selected_editor_item.item,
+                 selected_editor_item.variant,
+                 selected_editor_item.rotation);
     p_grf->redraw_add(&r);
   }
   {
@@ -443,9 +475,9 @@ void editor_gui::selection_draw(void)
   
     p_grf->fill(&r,0);
     p_font->print(&dr,r.x,r.y,_("Item: %s\nVariation: %d\nRotation: %s"),
-            p_repo->item_get_name(sel.item),
-            sel.variant,
-            p_repo->item_get_rotation(sel.rotation));
+            p_repo->item_get_name(selected_editor_item.item),
+            selected_editor_item.variant,
+            p_repo->item_get_rotation(selected_editor_item.rotation));
     r.h = dr.h;
     p_grf->redraw_add(&r);
   }  
@@ -493,34 +525,47 @@ void editor_gui::selection_cursor_update(void)
 
 /* Side menu - general
 */
-
-#define SIDE_MENU_X       (EDITOR_SCREEN_START_X+GAME_RESOLUTION_X+10)
-#define SIDE_MENU_Y       (EDITOR_SCREEN_START_Y+100)
-#define SIDE_MENU_DX      (EDITOR_RESOLUTION_X-SIDE_MENU_X)
-#define SIDE_MENU_DY      (EDITOR_RESOLUTION_Y-SIDE_MENU_Y)
-#define SIDE_MENU_X_DIFF   0
-#define SIDE_MENU_Y_DIFF   35
-
+#define SIDE_MENU_X         (EDITOR_SCREEN_START_X+GAME_RESOLUTION_X+10)
+#define SIDE_MENU_Y         (EDITOR_SCREEN_START_Y+60)
+#define SIDE_MENU_DX        (EDITOR_RESOLUTION_X-SIDE_MENU_X)
+#define SIDE_MENU_DY        (EDITOR_RESOLUTION_Y-SIDE_MENU_Y)
+#define SIDE_MENU_X_DIFF    0
+#define SIDE_MENU_Y_DIFF    35
 
 static char *side_menu[] = 
 { 
-  _("(1) floor"),
-  _("(2) items"),
-  _("(3) players"),
-  _("(4) all")
+  _("help (f1)"),
+  _("new level"),
+  _("save (f2)"),
+  _("save as (ct+f2)"),
+  _("load (f3)"),
+  _("quit (esc)"),
+  _("run level (f9)"),
+  _("undo (ctrl+u)"),
+  _("redo (ctrl+r)"),
+  _("rotate (shft+r)"),
+  _("shade floor"),
+  _("background (b)"),
 };
 
 void editor_gui::side_menu_create(void)
 {
-  p_font->print(NULL,SIDE_MENU_X, SIDE_MENU_Y, "Select layer:");
+  p_font->print(NULL,SIDE_MENU_X, SIDE_MENU_Y, "Editor menu:");
 
   menu_item_set_pos(SIDE_MENU_X, SIDE_MENU_Y+SIDE_MENU_Y_DIFF);
   menu_item_set_diff(SIDE_MENU_X_DIFF, SIDE_MENU_Y_DIFF);
 
-  menu_item_draw(side_menu[0], LEFT_NO_ARROW, TRUE, LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_FLOOR));
-  menu_item_draw(side_menu[1], LEFT_NO_ARROW, TRUE, LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_ITEMS));
-  menu_item_draw(side_menu[2], LEFT_NO_ARROW, TRUE, LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_PLAYER));
-  menu_item_draw(side_menu[3], LEFT_NO_ARROW, TRUE, LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,ALL_LEVEL_LAYERS));
+  menu_item_draw(side_menu[0], LEFT, TRUE, LEVEL_EVENT(ED_HELP));
+  menu_item_draw(side_menu[1], LEFT, TRUE, LEVEL_EVENT(ED_LEVEL_NEW));
+  menu_item_draw(side_menu[2], LEFT, TRUE, LEVEL_EVENT(ED_LEVEL_SAVE));
+  menu_item_draw(side_menu[3], LEFT, TRUE, LEVEL_EVENT(ED_LEVEL_SAVE_AS));
+  menu_item_draw(side_menu[4], LEFT, TRUE, LEVEL_EVENT(ED_LEVEL_LOAD));
+  menu_item_draw(side_menu[5], LEFT, TRUE, LEVEL_EVENT(ED_QUIT));
+  menu_item_draw(side_menu[6], LEFT, TRUE, LEVEL_EVENT(ED_LEVEL_RUN));
+  menu_item_draw(side_menu[7], LEFT, TRUE, LEVEL_EVENT(ED_UNDO));
+  menu_item_draw(side_menu[9], LEFT, TRUE, LEVEL_EVENT(ED_ROTATE_SELECTION));
+  menu_item_draw(side_menu[10],LEFT, TRUE, LEVEL_EVENT(ED_LEVEL_SHADER));
+  menu_item_draw(side_menu[11],LEFT, TRUE, LEVEL_EVENT(ED_LEVEL_CHANGE_BACKGROUND));
 
   p_grf->redraw_add(SIDE_MENU_X,SIDE_MENU_Y,SIDE_MENU_DX,SIDE_MENU_DY);
 }
@@ -528,9 +573,9 @@ void editor_gui::side_menu_create(void)
 #define SIDE_STATUS_X     (EDITOR_SCREEN_START_X+GAME_RESOLUTION_X+10)
 #define SIDE_STATUS_Y     (EDITOR_SCREEN_START_Y)
 #define SIDE_STATUS_DX    (EDITOR_RESOLUTION_X-SIDE_STATUS_X)
-#define SIDE_STATUS_DY    100
+#define SIDE_STATUS_DY    40
 
-void editor_gui::side_menu_update(void)
+void editor_gui::side_menu_draw(void)
 {
   p_font->select(FONT_DEFAULT);
   p_font->alignment_set(LEFT);
@@ -558,7 +603,7 @@ void editor_gui::side_menu_update(void)
       break;
   }
   if(p_act)
-    p_font->print(NULL,SIDE_STATUS_X,SIDE_STATUS_Y,_("active layer:\n\n%s"),p_act);
+    p_font->print(NULL,SIDE_STATUS_X,SIDE_STATUS_Y,_("edited layer:\n%s"),p_act);
   p_grf->redraw_add(SIDE_STATUS_X,SIDE_STATUS_Y,SIDE_STATUS_DX,SIDE_STATUS_DY);
 }
 
@@ -579,10 +624,10 @@ void editor_gui::layer_menu_draw(void)
 
   static char *layer_names[ALL_LEVEL_LAYERS] = 
   {   
-    _("(1)Grid:"),
-    _("(2)Floor:"),
-    _("(3)Items:"),
-    _("(4)Players:")
+    _("Grid:"),
+    _("Floor:"),
+    _("Items:"),
+    _("Players:")
   };
 
   static int layer_handle[] = 
@@ -658,7 +703,7 @@ void editor_gui::layer_status_switch(int layer, LAYER_STATE state)
 void editor_gui::layer_active_set(int layer)
 {
   config.lc.set_active(layer);  
-  side_menu_update();
+  side_menu_draw();
 }
 
 int editor_gui::layer_active_get(void)
@@ -751,7 +796,18 @@ void editor_gui::level_new_callback(void)
   }
 }
 
-void editor_gui::level_load(char *p_file, bool force)
+char *editor_gui::level_name_get(void)
+{
+  return(level_name);
+}
+
+void editor_gui::level_name_set(char *p_name)
+{
+  strncpy(level_name,p_name,MAX_FILENAME);  
+  level_caption_update();
+}
+
+void editor_gui::level_load(char *p_file, int force)
 {
   static char file[MAX_FILENAME] = "";
 
@@ -771,23 +827,25 @@ void editor_gui::level_load(char *p_file, bool force)
     }
   }
 
-  // Save undo
-  undo_store();  
-
-  // Load given level
-  char *p_paths[] = { p_dir->levels_user_get(), p_dir->cwd_get(), NULL };
-
-  bool loaded = level.level_load(p_file,p_paths,sizeof(p_paths)/sizeof(p_paths[0]));
-  if(loaded) {
-    strncpy(level_name,p_file,MAX_FILENAME);
-    level_edited_clear();
-    level_caption_update();
-  } else {
-    console.print(_("Unable to load level %s"),p_file);
+  // Do we have a valid filename?
+  if(p_file[0]) {
+    // Save undo
+    undo_store();
+  
+    // Load given level
+    const char *p_paths[] = { p_dir->levels_user_get(), p_dir->cwd_get(), NULL };
+  
+    bool loaded = level.level_load(p_file,p_paths,sizeof(p_paths)/sizeof(p_paths[0]));
+    if(loaded) {
+      level_name_set(p_file);
+      level_edited_clear();
+    } else {
+      console.print(_("Unable to load level %s"),p_file);
+    }
+  
+    // Clear internal name
+    file[0] = '\0';
   }
-
-  // Clear internal name
-  file[0] = '\0';
 }
 
 void editor_gui::level_load_callback(void)
@@ -806,7 +864,7 @@ void editor_gui::level_load_callback(void)
   }
 }
 
-void editor_gui::level_save_as(char *p_file, bool force)
+void editor_gui::level_save_as(char *p_file, int force)
 {
   static char file[MAX_FILENAME] = "";
   static int  force_saved = BOOL_UNDEFINED;
@@ -826,18 +884,21 @@ void editor_gui::level_save_as(char *p_file, bool force)
   }
 
   if(!force && level.level_exists(p_file)) {
-    input_start(&editor_gui::level_save_as_callback, HANDLE_2, INPUT_BOOLEAN, _("file %s exists! overwrite?:"));
+    input_start(&editor_gui::level_save_as_callback, HANDLE_2, INPUT_BOOLEAN, _("file %s exists! overwrite?:"),p_file);
     return;
-  }  
-
-  if(level.level_save(p_file)) {
-    level_edited_clear();
-  }
-  else {
-    console.print(_("Unable to save level %s"),p_file);
   }
 
-  console.print(_("Saved as %s"),p_file);
+  // Do we have a corect filename?
+  if(p_file[0]) {
+    if(level.level_save(p_file)) {
+      level_edited_clear();
+      level_name_set(p_file);
+    }
+    else {
+      console.print(_("Unable to save level %s"),p_file);
+    }
+    console.print(_("Saved as %s"),p_file);
+  }
   file[0] = '\0';
   force_saved = BOOL_UNDEFINED;
 }
@@ -858,15 +919,101 @@ void editor_gui::level_save_as_callback(void)
   }  
 }
 
-void editor_gui::level_save(bool force)
+void editor_gui::help_print_line(tpos x_pos, tpos &y_pos, char *p_key, char *p_name = NULL)
+{
+  p_font->print(NULL,x_pos,y_pos,p_key);
+  if(p_name) {
+    p_font->print(NULL,x_pos+EDIT_HELP_KEY_DX,y_pos,p_name);
+  }
+  y_pos += p_font->height_get();
+  y_pos += EDIT_HELP_DY;
+}
+
+void editor_gui::help(void)
+{
+  p_grf->fill(0,0,EDITOR_RESOLUTION_X,EDITOR_RESOLUTION_Y,0);
+  
+  p_font->select(FONT_DEFAULT);
+
+  tpos x_pos = EDIT_HELP_KEY_X;
+  tpos y_pos = EDIT_HELP_START_Y;
+
+  help_print_line(x_pos,y_pos,_("Keyboard control:"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("F1"),_("- Help"));
+  help_print_line(x_pos,y_pos,_("ESC"),_("- Quit"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("CTRL+N"),_("- New level"));
+  help_print_line(x_pos,y_pos,_("f2"),_("- Save level"));
+  help_print_line(x_pos,y_pos,_("CTRL+F2"),_("- Save level as"));
+  help_print_line(x_pos,y_pos,_("F3"),_("- Load level"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("F9"),_("- Run level"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("SHIFT+R"),_("- Rotate item"));
+  help_print_line(x_pos,y_pos,_("CTRL+U"),_("- Undo"));
+  help_print_line(x_pos,y_pos,_("CTRL+S"),_("- Shade level"));
+  help_print_line(x_pos,y_pos,_("B"),_("- Background"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("1"),_("- Select floor layer"));
+  help_print_line(x_pos,y_pos,_("2"),_("- Select items layer"));
+  help_print_line(x_pos,y_pos,_("3"),_("- Select players layer"));
+  help_print_line(x_pos,y_pos,_("4"),_("- Select all layer"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("CTRL+1"),_("- on/off background"));
+  help_print_line(x_pos,y_pos,_("CTRL+2"),_("- on/off floor layer"));
+  help_print_line(x_pos,y_pos,_("CTRL+3"),_("- on/off items layer"));
+  help_print_line(x_pos,y_pos,_("CTRL+4"),_("- on/off players layer"));
+
+  x_pos = EDIT_HELP_MOUSE_X;
+  y_pos = EDIT_HELP_START_Y;
+  help_print_line(x_pos,y_pos,_("Mouse control:"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("first"),_("- insert selected item"));
+  help_print_line(x_pos,y_pos,_("third"),_("- clear selected cell"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("R+wheel"),_("- in place rotation"));
+  help_print_line(x_pos,y_pos,_("V+wheel"),_("- in place variation"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("F+first"),_("- fill rect with item"));
+  help_print_line(x_pos,y_pos,_("D+first"),_("- draw rect with item"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("F+third"),_("- clear solid rect"));
+  help_print_line(x_pos,y_pos,_("D+third"),_("- clear empty rect"));
+
+  x_pos = EDIT_HELP_MOUSE_X+70;
+  y_pos += EDIT_HELP_DY*6;
+  help_print_line(x_pos,y_pos,_("item panel mouse control:"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("wheel"),_("- scroll by one"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("PgUp"),_("- scroll page up"));
+  help_print_line(x_pos,y_pos,_("PgDown"),_("- scroll page down"));
+  y_pos += EDIT_HELP_DY;
+  help_print_line(x_pos,y_pos,_("Home"),_("- first item"));
+  help_print_line(x_pos,y_pos,_("End"),_("- last item"));
+
+  p_grf->redraw_add(0,0,EDITOR_RESOLUTION_X,EDITOR_RESOLUTION_Y);
+
+  menu_key_input.set((GUI_BASE *)this,(GUI_BASE_FUNC)&editor_gui::console_wait);
+  console.input_start(INPUT_WAIT, NULL);  
+  
+  draw_level = FALSE;
+}
+
+void editor_gui::help_quit(void)
+{
+  editor_reset();
+}
+
+void editor_gui::level_save(int force)
 {
   level_save_as(level_name,force);
 }
 
 void editor_gui::level_cursor_set(tpos x, tpos y)
 {
-  bool state;
-  
+  bool state;  
 
   if((state = level.coord_in_level(x,y))) {
     level.coord_to_cell(&x,&y);
@@ -882,7 +1029,7 @@ void editor_gui::level_item_insert(void)
   if(level.selection_get()) {
     undo_store();
   
-    tpos layer = p_repo->item_get_layer(sel.item);
+    tpos layer = p_repo->item_get_layer(selected_editor_item.item);
   
     bool rect = level.selection_rectangle_get();  
     if(rect) {
@@ -890,18 +1037,27 @@ void editor_gui::level_item_insert(void)
       tpos sx,sy,dx,dy;
     
       level.selection_rectangle_get(&sx,&sy,&dx,&dy);
-      level.cell_set(sx, sy, dx, dy, layer, sel.item, sel.variant, sel.rotation, TRUE, fill);
+      level.cell_set(sx, sy, dx, dy, layer, 
+                     selected_editor_item.item, 
+                     selected_editor_item.variant, 
+                     selected_editor_item.rotation, 
+                     TRUE, fill);
     
       level.selection_rectangle_set(FALSE);
-      console.print(_("Inserted item %d at %dx%d - %dx%d layer %d"),sel.item,sx,sy,dx,dy,layer);
+      console.print(_("Inserted item %d at %dx%d - %dx%d layer %d"),
+                    selected_editor_item.item,sx,sy,dx,dy,layer);
     }
     else {
       tpos x, y;
     
       level.selection_get(&x, &y);
-      level.cell_set(x, y, layer, sel.item, sel.variant, sel.rotation, TRUE);
+      level.cell_set(x, y, layer, 
+                     selected_editor_item.item, 
+                     selected_editor_item.variant, 
+                     selected_editor_item.rotation, TRUE);
     
-      console.print(_("Inserted item %d at %dx%d layer %d"),sel.item,x,y,layer);
+      console.print(_("Inserted item %d at %dx%d layer %d"),
+                    selected_editor_item.item,x,y,layer);
     }
   
     level_edited_set();
@@ -964,7 +1120,7 @@ void editor_gui::level_item_variate(CHANGE_TYPE type, int variant)
       if(type == ITEM_MODIFY) {
         level.cell_modify_variation(x,y, layer, variant, TRUE, TRUE);
         console.print(_("[Modify] Variating item at %dx%d layer %d"),x,y,layer);
-      } 
+      }
       else if(type == ITEM_SET) {
         level.cell_set_variation(x,y, layer, variant, TRUE, TRUE);
         console.print(_("[Set] Variating item at %dx%d layer %d"),x,y,layer);
@@ -1017,11 +1173,11 @@ void editor_gui::level_item_clear_rectangle(bool filled)
 
 void editor_gui::selection_rotate(int direction)
 {
-  if(p_repo->item_can_rotate(sel.item)) {
-    ROTATE_ITEM(sel.rotation, direction);
+  if(p_repo->item_can_rotate(selected_editor_item.item)) {
+    ROTATE_ITEM(selected_editor_item.rotation, direction);
     selection_draw();
   } else {
-    console.print(_("Can't rotate item %d"),sel.item);
+    console.print(_("Can't rotate item %d"),selected_editor_item.item);
   }
 }
 
@@ -1033,7 +1189,7 @@ void editor_gui::mouse_handler(LEVEL_EVENT_QUEUE *p_queue, LEVEL_EVENT ev)
   switch(ev.param_int_get(PARAM_0)) {
     case SCREEN_HANDLE_IN:
       p_queue->add(LEVEL_EVENT(ED_LEVEL_SET_CURSOR,ev.param_get(PARAM_1),ev.param_get(PARAM_2),ET(TRUE)));
-      p_queue->commit();  
+      p_queue->commit();
       break;
     case SCREEN_HANDLE_OUT:
       p_queue->add(LEVEL_EVENT(ED_LEVEL_SET_CURSOR,-1,-1,TRUE));
@@ -1045,19 +1201,19 @@ void editor_gui::mouse_handler(LEVEL_EVENT_QUEUE *p_queue, LEVEL_EVENT ev)
       if(button == BUTTON_LEFT)
         p_queue->add(LEVEL_EVENT(ED_LEVEL_DRAW_CURSOR_INSERT_ITEM));
       else if(button == BUTTON_MIDDLE)
-        ;        
+        ;
       else if(button == BUTTON_RIGHT)
-        p_queue->add(LEVEL_EVENT(ED_LEVEL_DRAW_CURSOR_CLEAR_ITEM));        
-      p_queue->commit();      
+        p_queue->add(LEVEL_EVENT(ED_LEVEL_DRAW_CURSOR_CLEAR_ITEM));
+      p_queue->commit();
       break;
     case SCREEN_HANDLE_ROTATE:
-      direction = (ev.param_int_get(PARAM_4) == WHEEL_UP) ? -1 : 1;  
+      direction = (ev.param_int_get(PARAM_4) == WHEEL_UP) ? -1 : 1;
       p_queue->add(LEVEL_EVENT(ED_LEVEL_SET_CURSOR,ev.param_get(PARAM_1),ev.param_get(PARAM_2)));
       p_queue->add(LEVEL_EVENT(ED_LEVEL_DRAW_CURSOR_ROTATE_ITEM, ITEM_MODIFY, direction));
       p_queue->commit();
       break;
     case SCREEN_HANDLE_VARIATE:
-      direction = (ev.param_int_get(PARAM_4) == WHEEL_UP) ? -1 : 1;    
+      direction = (ev.param_int_get(PARAM_4) == WHEEL_UP) ? -1 : 1;
       p_queue->add(LEVEL_EVENT(ED_LEVEL_SET_CURSOR,ev.param_get(PARAM_1),ev.param_get(PARAM_2)));
       p_queue->add(LEVEL_EVENT(ED_LEVEL_DRAW_CURSOR_VARIATE_ITEM,ITEM_MODIFY,direction));
       p_queue->commit();
@@ -1104,7 +1260,7 @@ void editor_gui::editor_run_level(void)
   return_path(p_dir->tmp_get(), TMP_LEVEL, filename, MAX_FILENAME);
 
   if(level.level_save(filename)) {
-    bprintf("Saved as %s",filename);
+#ifdef LINUX
     int pid = fork();
     if(!pid) {
       char level_name[MAX_FILENAME];
@@ -1121,6 +1277,16 @@ void editor_gui::editor_run_level(void)
       waitpid(pid,&status,0);
       bprintf("Pid %d done",pid);
     }
+#elif WINDOWS  
+    bprintf("Saved as %s",filename);
+    char level_name[MAX_FILENAME];
+    return_path(p_dir->tmp_get(), TMP_LEVEL, level_name, MAX_FILENAME);
+    bprintf("%s -u %s",p_dir->game_binary_get(),level_name);    
+    int ret = _spawnl( _P_WAIT, p_dir->game_binary_get(),p_dir->game_binary_get(),"-u",level_name,NULL);
+    if(ret == -1) {
+      bprintf("Error: %s",strerror(errno));
+    }
+#endif  
   }
 }
 
@@ -1228,7 +1394,16 @@ bool editor_gui::event_handler(void)
         case EV_TEST:
           test();
           break;
-      
+
+        case ED_HELP:
+          // Clear all waiting events
+          queue.clear();
+          tmp_queue.clear();
+        
+          // Print help page
+          help();        
+          break;
+        
         case ED_LEVEL_NEW:
           level_new(ev.param_int_get(PARAM_0));
           break;
@@ -1270,6 +1445,10 @@ bool editor_gui::event_handler(void)
         case ED_LEVEL_IPANEL_SCROLL:
           panel_scroll(ev.param_int_get(PARAM_0), ev.param_int_get(PARAM_1));
           break;
+        
+        case ED_LEVEL_MOUSE_PANEL_SCROLL:
+          panel_scroll_mouse(ev.param_int_get(PARAM_0));
+          break;
   
         case ED_LEVEL_LAYER:        
           layer_status_switch(ev.param_int_get(PARAM_0),(LAYER_STATE)(ev.param_int_get(PARAM_1)));
@@ -1287,7 +1466,7 @@ bool editor_gui::event_handler(void)
           undo_restore();
           break;
         case ED_REDO:
-          break;        
+          break;
         
         case ED_LEVEL_RUN:
           editor_run_level();
@@ -1339,8 +1518,10 @@ bool editor_gui::event_handler(void)
   
   } while(!queue.empty());
 
-  // Draw all changes    
-  level.draw();
+  // Draw all changes in level
+  if(draw_level) {
+    level.draw();
+  }
   level.flip();
 
   return(TRUE);
@@ -1445,15 +1626,29 @@ void editor_gui::input_stop(bool success)
 {
   menu_key_input.clear();
   console.input_stop();
-  (this->*console_callback)();
+  if(console_callback) {
+    (this->*console_callback)();
+  }
+}
+
+/* Wait for a key
+*/
+void editor_gui::console_wait(MENU_STATE state, int data, int data1)
+{ 
+  switch(data) {
+    default:
+      help_quit();
+      input_stop(TRUE);
+      break;
+  }
 }
 
 // ----------------------------------------------------------------------
 //  An editor console code
 // ----------------------------------------------------------------------
 
-char * editor_console::boolean_yes = _("y");
-char * editor_console::boolean_no = _("n");
+const char * editor_console::boolean_yes = _("y");
+const char * editor_console::boolean_no = _("n");
 
 editor_console::editor_console(INPUT *p_input_, tpos sx, tpos sy, tpos dx, int lines_)
 {
@@ -1490,9 +1685,12 @@ void editor_console::input_start(INPUT_TYPE type, char *p_text)
   if(itype == INPUT_BOOLEAN) {
     sprintf(input_line_title+strlen(input_line_title)," (%s/%s) ",
             boolean_yes,boolean_no);
-  } else {
+  } 
+  else if(itype == INPUT_STRING) {
     sprintf(input_line_title+strlen(input_line_title)," ");
   }
+  else if(type == INPUT_WAIT) {
+  }
 
   p_input->block(TRUE);
   p_input->key_repeat(TRUE);
@@ -1503,7 +1701,7 @@ void editor_console::input_start(INPUT_TYPE type, char *p_text)
 void editor_console::input_stop(bool redraw)
 {  
   p_input->block(FALSE);
-  p_input->key_repeat(FALSE);  
+  p_input->key_repeat(FALSE);
 
   istate = INPUT_NONE;
   input_redraw();
@@ -1544,7 +1742,7 @@ void editor_console::input_add_char(char c, bool redraw)
       case INPUT_BOOLEAN:
         if(c == boolean_yes[0]) {
           input_boolean = TRUE;
-        }
+        }      
         else if(c == boolean_no[0]) {
           input_boolean = FALSE;
         }
@@ -1566,7 +1764,10 @@ void editor_console::input_add_char(char c, bool redraw)
 
 void editor_console::input_redraw(void)
 { 
-  if(istate == INPUT_READY) {  
+  if(itype == INPUT_WAIT) {
+    // nothing here...
+  } 
+  else if(istate == INPUT_READY) {  
     p_grf->fill(ix_title,iy_title,w,CONSOLE_INPUT_LINE_HEIGHT,COLOR_MAP(0,0,255));
 
     p_font->select(FONT_DEFAULT);
@@ -1579,7 +1780,7 @@ void editor_console::input_redraw(void)
     iy_input = iy_title;
       
     if(itype == INPUT_BOOLEAN) {
-      char *p_tmp;
+      const char *p_tmp;
     
       if(input_boolean == CONSOLE_BOOLEAN_NO_INPUT)
         p_tmp = "_";
@@ -1600,13 +1801,13 @@ void editor_console::input_redraw(void)
   p_grf->redraw_add(ix_title,iy_title,w,CONSOLE_INPUT_LINE_HEIGHT);
 }
 
-void editor_console::print(char *p_text,...)
+void editor_console::print(const char *p_text,...)
 {
   strncpy(output_lines[0],output_lines[1],CONSOLE_MAX_INPUT_LINE);
   
   va_list   arguments;
   va_start(arguments,p_text);
-  vsnprintf(output_lines[1]+2,CONSOLE_MAX_INPUT_LINE,p_text,arguments);
+  vsnprintf(output_lines[1]+2,CONSOLE_MAX_INPUT_LINE-2,p_text,arguments);
   va_end(arguments);  
   output_lines[1][0] = '*';
   output_lines[1][1] = ' ';
