@@ -29,7 +29,6 @@
 #include <stdlib.h>
 #include <time.h>
 #include <string.h>
-#include <curses.h>
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -39,13 +38,6 @@
 #include "main.h"
 #include "editor.h"
 
-#ifdef HILDON
-#define HAVE_OSSO
-#include <libosso.h>
-// #include <dbus/dbus-glib-lowlevel.h>
-// #include "dbus-print-message.h"
-#endif
-
 /* Time loop - grabbed from SDL documentation */
 #define  TICK_INTERVAL    (1000 / GAME_FPS)
 
@@ -62,7 +54,7 @@ Uint32 time_left(void)
 
 void banner(void)
 {
-  printf(_("Berusky v.%s (C) Anakreon 2006, http://www.anakreon.cz/\n"),VERSION);
+  printf(_("Berusky v.%s (C) Anakreon 2006, http://www.anakreon.cz/\n"),BERUSKY_VERSION);
   printf(_("This is free software; see the source for copying conditions.\n"));
   printf(_("There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"));
   printf(_("Built %s, %s\n\n"),__DATE__,__TIME__);
@@ -77,7 +69,6 @@ void help(bool error)
   printf(_("  -e [level.lv3]       -run level editor\n"));
   printf(_("  -u  level.lv3        -run level\n"));
 /* TODO - not implemented yet
-  printf(_("Using: berusky [-e [level.lv3]] [-u level.lv3] [-d level.dm]\n\n"));
   printf(_("  -d  level.dm         -run demo.dm\n"));
 */  
   exit(0);
@@ -242,10 +233,10 @@ void run_editor(GAME_MODE gmode, char *p_garg, DIR_LIST *p_dir)
 /*
  * Manage a configuration file
  */
-char * config_file(bool configure)
+const char * config_file(bool configure)
 {
-  static char *files[] = { INI_FILE_LOCAL, INI_FILE_USER, INI_FILE_GLOBAL };
-  static char *ini_file = NULL;
+  static const char *files[] = { INI_FILE_LOCAL, INI_FILE_USER, INI_FILE_GLOBAL };
+  static const char *ini_file = NULL;
 
   if(configure) {
     int i;
@@ -294,7 +285,6 @@ void user_directory_create(void)
   bprintf(" ");
 }
 
-
 /*
  * Let's begin
  */
@@ -307,11 +297,6 @@ int main(int argc, char *argv[])
   setbuf(stderr, NULL);
   srand(clock());
   
-#ifdef HILDON
-  // to create an entry in the task list
-  putenv("SDL_VIDEO_X11_WMCLASS=berusky");
-#endif
-
   banner();
   
   /* Run help */
@@ -346,25 +331,6 @@ int main(int argc, char *argv[])
     help(TRUE);
   }
 
-#ifdef HILDON	
-	// 2nd try to set the title in the task bar
-  // SDL_WM_SetCaption("Berusky",NULL);
-
-#ifdef HAVE_OSSO	
-	/* LibOSSO context object/connection */
-	osso_context_t* osso_ctx = NULL;
-  osso_return_t ores;
-  printf("Initializing LibOSSO context (net.sourceforge.berusky)\n");
-	osso_ctx = osso_initialize("net.sourceforge.berusky", "1.1", TRUE, NULL);
-	if(osso_ctx == NULL) {
-	  printf("Failed to init LibOSSO\n");
-	  return 1;
-	}
-	printf("LibOSSO Init done\n");
-#endif // HAVE_OSSO
-
-#endif // HILDON
-
   user_directory_create();
   config_file(TRUE);
   log_open_ini(INI_FILE);
@@ -377,12 +343,7 @@ int main(int argc, char *argv[])
   } else {
     run_game(gmode, p_garg, &dir);
   }  
-
-#ifdef HAVE_OSSO
-  if(osso_ctx != NULL) 
-    osso_deinitialize(osso_ctx);
-#endif
-
+  return 0;
 }
 
 void start_logo_draw(GAME_MODE mode)
@@ -448,7 +409,7 @@ bool graphics_logos_load(DIR_LIST *p_dir)
   p_grf->graphics_dir_set(p_dir->graphics_get());
 
   sprite::color_key_set(COLOR_KEY_GAME);
-  i  += p_grf->sprite_insert("logo.spr",   FIRST_LOGO, 1, NULL);
+  i  += p_grf->sprite_insert("logo.spr", FIRST_LOGO, 1, NULL);
 
   return(i);
 }
